#!perl
#
# Tests to check viewports

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";
use lib "/usr/local/nagios/lib";

use Test::WWW::Mechanize;
use HTTP::Cookies;
use Sys::Hostname;
use Data::Dump qw(dump);
use Opsview::Utils;

use Opsview::Web::Test;
use Opsview::TestUtils;

use Opsview;
use Opsview::Contact;

use Test::More qw(no_plan);
use Test::Deep;

use Opsview::Schema;

my $schema = Opsview::Schema->my_connect;

use Runtime::Schema;
my $runtime = Runtime::Schema->my_connect;

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

my $public = Test::WWW::Mechanize->new();

my $admin = Test::WWW::Mechanize->new;
log_in( $admin, 'admin', 'initial', "http://$hostname" );

my $somekeywords = Test::WWW::Mechanize->new;
log_in( $somekeywords, 'viewkeywords2', 'viewkeywords2', "http://$hostname" );

my $allkeywords = Test::WWW::Mechanize->new;
log_in( $allkeywords, 'keywordonly', 'keywordonly', "http://$hostname" );

my $somehosts = Test::WWW::Mechanize->new;
log_in( $somehosts, "somehosts", "somehosts", "http://$hostname" );

my $res;
$res = $public->get("http://$hostname/viewport/cisco_gp1");
ok( $res->is_success, "Got viewport page, without auth required" );
is( $res->code, 200, "Status 200" );
like( $res->content, qr%<title>Opsview &gt; Viewport &gt; cisco devices, group 1</title>%, "Returned 'Status for' header" );
like( $res->content, '/id="main_content"/', "Got main_content id" );

$public->content_contains( "Loading...", "Got loading text" );
$public->get_ok("/viewport/cisco_gp1?output=ajax", "Got the ajax call to update the main content");

my $link = $public->find_link( url => "http://opsview1.org" );
isa_ok( $link, "WWW::Mechanize::Link", "Which means markdown conversion is working" );

$link = $public->find_link( url => "http://opsview2.org" );
is( $link, undef, "No link to opsview2.org" );
$public->content_contains( "http://opsview2.org", "...even though the string exists" );

$res = $public->get("/viewport/cisco_gp1?style=group_by_service&output=ajax");
$link = $public->find_link( url => "http://opsview1.org" );
isa_ok( $link, "WWW::Mechanize::Link", "Which means markdown conversion is working" );

$link = $public->find_link( url => "http://opsview2.org" );
is( $link, undef, "No link to opsview2.org" );
$public->content_contains( "http://opsview2.org", "...even though the string exists" );

$res = $public->get("/viewport/cisco");
ok( $res->is_success, "Got viewport page for cisco keyword" );
like( $res->content, qr%<title>Opsview &gt; Viewport &gt; cisco devices</title>%, "Got header" );

$public->get("/viewport/cisco?output=ajax");
$public->content_contains( "administrator: This is acknowledged", "Got acknowledgement" );

$res = $public->get("/viewport");
ok( $res->is_success, "Got viewport summary page" );
like( $res->content, "/<title>Opsview &gt; Viewport Summary</title>/", "Got title" );
$_ = $res->content;

# This test is not valid as viewport summary page does not show actual summary data any more, but has a loading page
# which then brings it in via ajax
#my $num = s/<tr//g;
#is( $num, 4, "Four rows found: 1 header + 3 data" );
like( $_, qr/Loading.../, "Has a loading page" );

$public->content_lacks( "alphaearly", "This keyword should not be listed as there are no services associated to this viewport" );
$public->content_lacks( "cisco_gp2", "This keyword should not be listed as is not a public viewport" );



# Check public viewports
my $expected = {
    'ResultSet' => {
        'summary' => {
            'handled'   => 9,
            'unhandled' => 23,
            'total'     => 32,
            'service'   => {
                'handled'   => 1,
                'unknown'   => 24,
                'unhandled' => 23,
                'total'     => 24
            },
            'host' => {
                'handled'   => 8,
                'unhandled' => 0,
                'up'        => 8,
                'total'     => 8
            }
        },
        'list' => [
            {   'hosts' => {
                    'handled'   => 5,
                    'unhandled' => 0,
                    'up'        => { 'handled' => 5 },
                    'total'     => 5
                },
                'services' => {
                    'handled' => 1,
                    'computed_state' => 'unknown',
                    'unknown' => {
                        'handled'   => 1,
                        'unhandled' => 14
                    },
                    'unhandled' => 14,
                    'total'     => 15
                },
                'computed_state' => 'unknown',
                'name'        => 'cisco',
                'downtime'    => 2,
                'description' => 'cisco devices'
            },
            {   'hosts' => {
                    'handled'   => 3,
                    'unhandled' => 0,
                    'up'        => { 'handled' => 3 },
                    'total'     => 3
                },
                'services' => {
                    'handled'   => 0,
                    'computed_state'   => 'unknown',
                    'unknown'   => { 'unhandled' => 9 },
                    'unhandled' => 9,
                    'total'     => 9
                },
                'computed_state' => 'unknown',
                'name'        => 'cisco_gp1',
                'downtime'    => 2,
                'description' => 'cisco devices, group 1'
            },
        ]
    }
};

$res = $public->get("http://$hostname/viewport?output=json&keyword=cisco&keyword=cisco_gp1&keyword=cisco_gp2");
ok( $res->is_success, "Got viewport summary page in json output" );
use JSON;
my $obj = from_json( $res->content );
is_deeply( $obj, $expected, "Object as expected" ) || diag explain $obj;


# Check user access
$expected = {
    'ResultSet' => {
        'summary' => {
            'handled'   => 13,
            'unhandled' => 31,
            'total'     => 44,
            'service'   => {
                'handled'   => 2,
                'unknown'   => 33,
                'unhandled' => 31,
                'total'     => 33
            },
            'host' => {
                'handled'   => 11,
                'unhandled' => 0,
                'up'        => 11,
                'total'     => 11
            }
        },
        'list' => [
            {   'hosts' => {
                    'handled'   => 5,
                    'unhandled' => 0,
                    'up'        => { 'handled' => 5 },
                    'total'     => 5
                },
                'services' => {
                    'handled' => 1,
                    'computed_state' => 'unknown',
                    'unknown' => {
                        'handled'   => 1,
                        'unhandled' => 14
                    },
                    'unhandled' => 14,
                    'total'     => 15
                },
                'computed_state' => 'unknown',
                'name'        => 'cisco',
                'downtime'    => 2,
                'description' => 'cisco devices'
            },
            {   'hosts' => {
                    'handled'   => 3,
                    'unhandled' => 0,
                    'up'        => { 'handled' => 3 },
                    'total'     => 3
                },
                'services' => {
                    'handled' => 0,
                    'computed_state' => 'unknown',
                    'unknown' => {
                        'unhandled' => 9
                    },
                    'unhandled' => 9,
                    'total'     => 9
                },
                'computed_state' => 'unknown',
                'name'        => 'cisco_gp1',
                'downtime'    => 2,
                'description' => 'cisco devices, group 1'
            },
            {   'hosts' => {
                    'handled'   => 3,
                    'unhandled' => 0,
                    'up'        => { 'handled' => 3 },
                    'total'     => 3
                },
                'services' => {
                    'handled'   => 1,
                    'computed_state'   => 'unknown',
                    'unknown'   => { handled => 1, 'unhandled' => 8 },
                    'unhandled' => 8,
                    'total'     => 9
                },
                'computed_state' => 'unknown',
                'name'        => 'cisco_gp2',
                'downtime'    => 2,
                'description' => 'Not public keyword',
            },
        ]
    }
};

$somekeywords->get_ok("/viewport?output=json&keyword=cisco&keyword=cisco_gp1&keyword=cisco_gp2");
$obj = from_json( $somekeywords->content );
is_deeply( $obj, $expected, "Should list all 3 viewports, as somekeywords2 has access to cisco and cisco_gp2, and cisco_gp1 is a public viewport") || diag explain $obj;


# Check that still cannot see viewport
$res = $public->get("http://$hostname/viewport/cisco_gp2");
ok( $res->is_success, "Got viewport page but no data in it" );
is( $res->code, 200, "Status 200" );
$public->content_lacks( "Not public keyword", "Not loaded with header" );
my $missing_cisco_gp2_error = "Keyword cisco_gp2 not found";
$public->content_contains( $missing_cisco_gp2_error, "Error message displayed because no public access available" );


# Confirm that logged in user sees more
$expected = {
    'ResultSet' => {
        'summary' => {
            'handled'   => 13,
            'unhandled' => 31,
            'total'     => 44,
            'service'   => {
                'handled'   => 2,
                'unknown'   => 33,
                'unhandled' => 31,
                'total'     => 33
            },
            'host' => {
                'handled'   => 11,
                'unhandled' => 0,
                'up'        => 11,
                'total'     => 11
            }
        },
        'list' => [
            {   'hosts' => {
                    'handled'   => 5,
                    'unhandled' => 0,
                    'up'        => { 'handled' => 5 },
                    'total'     => 5
                },
                'services' => {
                    'handled' => 1,
                    'computed_state' => 'unknown',
                    'unknown' => {
                        'handled'   => 1,
                        'unhandled' => 14
                    },
                    'unhandled' => 14,
                    'total'     => 15
                },
                'computed_state' => 'unknown',
                'name'        => 'cisco',
                'downtime'    => 2,
                'description' => 'cisco devices'
            },
            {   'hosts' => {
                    'handled'   => 3,
                    'unhandled' => 0,
                    'up'        => { 'handled' => 3 },
                    'total'     => 3
                },
                'services' => {
                    'handled'   => 0,
                    'computed_state'   => 'unknown',
                    'unknown'   => { 'unhandled' => 9 },
                    'unhandled' => 9,
                    'total'     => 9
                },
                'computed_state' => 'unknown',
                'name'        => 'cisco_gp1',
                'downtime'    => 2,
                'description' => 'cisco devices, group 1'
            },
            {   'hosts' => {
                    'handled'   => 3,
                    'unhandled' => 0,
                    'up'        => { 'handled' => 3 },
                    'total'     => 3
                },
                'services' => {
                    'handled' => 1,
                    'computed_state' => 'unknown',
                    'unknown' => {
                        'handled'   => 1,
                        'unhandled' => 8
                    },
                    'unhandled' => 8,
                    'total'     => 9
                },
                'computed_state' => 'unknown',
                'name'        => 'cisco_gp2',
                'downtime'    => 2,
                'description' => 'Not public keyword',
            }
        ]
    }
};

$res = $admin->get("http://$hostname/viewport?output=json&keyword=cisco&keyword=cisco_gp1&keyword=cisco_gp2");
ok( $res->is_success, "Got viewport summary page in json output for logged in user" );
$obj = from_json( $res->content );
is_deeply( $obj, $expected, "Object as expected - includes cisco_gp2 for logged in user" );

$allkeywords->get_ok("/viewport?output=json&keyword=cisco&keyword=cisco_gp1&keyword=cisco_gp2");
$obj = from_json( $allkeywords->content );
is_deeply( $obj, $expected, "Object as expected for keywordonly user too as has all_keywords in role access") || diag explain $obj;


$admin->get_ok("/viewport");
$admin->content_contains( "cisco_gp2", "cisco_gp2 should be listed as logged in users can see non-public keywords" );

$admin->get_ok("/viewport/cisco_gp2", "Can load cisco_gp2 for auth user");
$admin->content_lacks($missing_cisco_gp2_error, "No error message this time" );
$admin->content_contains("Not public keyword", "Details found for this keyword" );

$allkeywords->get_ok("/viewport");
$allkeywords->content_lacks( "alphaearly", "alphaearly keyword should be missing as no services associated" );

$allkeywords->get_ok("/viewport?keyword=addedunexpectedkeyword");
$allkeywords->content_contains(q{url: '\\/viewport?output=ajax',}, "Check that url does not embed the keyword portions");

$expected = {
   ResultSet => {
         list => [
               {
                 alias => "cisco4",
                 "current_check_attempt" => 0,
                 downtime => 2,
                 downtime_comment => "Hostgroup 'Monitoring Servers': testing",
                 downtime_username => "admin",
                 icon => "cisco",
                 last_check => "1970-01-01 00:00:00",
                 max_check_attempts => 0,
                 name => "cisco4",
                 output => "Dummy output",
                 num_interfaces => 1,
                 num_services => 3,
                 services => [
                       {
                         acknowledged => 1,
                         acknowledged_author => "administrator",
                         acknowledged_comment => "This is acknowledged",
                         "current_check_attempt" => 1,
                         downtime => 2,
                         downtime_comment => "Hostgroup 'Monitoring Servers': testing",
                         downtime_username => "admin",
                         last_check => "2007-10-31 13:20:40",
                         markdown => 0,
                         max_check_attempts => 3,
                         metrics => [{ name => "size" }, { name => "time" }],
                         name => "Another exception",
                         output => "forced result",
                         perfdata_available => 0,
                         service_object_id => 160,
                         state => "unknown",
                         unhandled => 0,
                       },
                       {
                         "current_check_attempt" => 1,
                         downtime => 2,
                         downtime_comment => "Hostgroup 'Monitoring Servers': testing",
                         downtime_username => "admin",
                         last_check => "2007-10-31 13:20:40",
                         markdown => 0,
                         max_check_attempts => 3,
                         metrics => [
                               { name => "losspct", uom => "", value => 0 },
                               { name => "rta", uom => "ms", value => 25 },
                             ],
                         name => "Coldstart",
                         output => "forced result",
                         perfdata_available => 0,
                         service_object_id => 161,
                         state => "unknown",
                         unhandled => 1,
                       },
                     ],
                 state => "up",
                 summary => { handled => 1, computed_state => "unknown", total => 2, unhandled => 1, unknown => 2 },
                 unhandled => 0,
               },
             ],
         summary => {
               handled   => 2,
               host      => { handled => 1, total => 1, unhandled => 0, up => 1 },
               service   => { handled => 1, total => 2, unhandled => 1, unknown => 2 },
               total     => 3,
               unhandled => 1,
             },
       },
};
$res = $public->get("http://$hostname/viewport/cisco?output=json&style=group_by_host&includeperfdata=1");
ok( $res->is_success, "Got the data for this viewport" );
$obj = from_json( $res->content );
Opsview::Utils->remove_keys_from_hash( $obj, ["state_duration"] );
is_deeply( $obj, $expected, "Object as expected - this URL is used by performance viewport" ) || diag(Data::Dump::dump($obj));


# Need to enable the viewport for the "disabled" keyword
$admin->get_ok("/admin/keyword/edit/5");
$admin->form_name("main_form");
$admin->field( "enabled", 1 );
$admin->submit_form;

$expected = {
   ResultSet => {
     list => [
       {
         alias                 => "Slave",
         current_check_attempt => 0,
         downtime              => 0,
         icon                  => "opsview",
         last_check            => "1970-01-01 00:00:00",
         max_check_attempts    => 0,
         name                  => "opslave",
         num_interfaces        => 0,
         num_services          => 6,
         output                => "Dummy output",
         services              => [
                                    {
                                      current_check_attempt => 1,
                                      downtime => 0,
                                     last_check => "2007-10-31 13:20:10",
                                      markdown => 0,
                                      max_check_attempts => 3,
                                      metrics => [{ name => "root", uom => "MB", value => 1004 }],
                                      name => "/",
                                      output => "DISK OK - free space: / 18385 MB (94% inode=-):",
                                     perfdata_available => 1,
                                      service_object_id => 197,
                                      state => "ok",
                                      unhandled => 0,
                                   },
                                  ],
         state                 => "up",
         summary               => { handled => 1, computed_state => "ok", ok => 1, total => 1, unhandled => 0 },
        unhandled             => 0,
       },
     ],
    summary => {
       handled   => 2,
       host      => { handled => 1, total => 1, unhandled => 0, up => 1 },
       service   => { handled => 1, ok => 1, total => 1, unhandled => 0 },
       total     => 2,
       unhandled => 0,
   },
  },
} ;
$res = $public->get("/viewport/disabled?output=json&style=group_by_host&includeperfdata=1");
ok( $res->is_success, "Got the data for this viewport" );
$obj = from_json( $res->content );
Opsview::Utils->remove_keys_from_hash( $obj, ["state_duration"] );
is_deeply( $obj, $expected, "Object as expected with funny perf data" ) || diag(Data::Dump::dump($obj));


$somekeywords->get_ok("/viewport/cisco_gp1");
$somekeywords->content_contains("Viewport &gt; cisco devices, group 1", "Can see detailed page as public viewport" );

# Going to remove public setting on cisco keyword and cisco_gp1
# Then prove viewkeywords2 can see cisco due to permission and not cisco_gp1
$schema->resultset("Keywords")->search( { name => [ "cisco", "cisco_gp1" ] } )->update( { public => 0 } );

$somekeywords->get_ok("/viewport/cisco");
$somekeywords->content_contains( "cisco devices", "Can still see cisco keyword due to access level" );
$somekeywords->content_lacks("not found", "No error message" );

$somekeywords->get_ok("/viewport/cisco_gp1");
$somekeywords->content_lacks( "cisco devices, group 1", "...but not cisco_gp1" );
$somekeywords->content_contains("not found", "Error message due to no access at keyword" );

$somekeywords->get_ok("/viewport");
$somekeywords->content_lacks("cisco_gp1", "Should not have cisco_gp1 listed");

$somekeywords->get_ok("/viewport?output=json");
$somekeywords->content_lacks("cisco_gp1", "Also should not have cisco_gp1 listed");


# Strange test for listing of keywords based on public or role access
$somehosts->get_ok("/viewport?output=json");
$somehosts->content_lacks("cisco_gp1", "Also should not have cisco_gp1 listed");

$somehosts->get_ok("/viewport?output=json");
$somehosts->content_lacks("cisco", "Also should not have cisco listed (could be substring)");

# Need to enable the viewport cisco to be public
$schema->resultset("Keywords")->search( { name => [ "cisco" ] } )->update( { public => 1 } );

$somehosts->get_ok("/viewport?output=json");
$somehosts->content_contains("cisco", "But comes back now due to being public") || diag $somehosts->content;

$somehosts->get_ok("/viewport");
$somehosts->content_contains("cisco", "Should be listed in menu" );


$somehosts->get_ok("/viewport?output=ajax&style=tagcloud");
$somehosts->content_contains('<span class="tagcloud3"><a href="/viewport/cisco" class="unknown" title="Unhandled failures: 14">cisco</a></span>',
    "Check for tagcloud-ness" ) || diag $admin->content;

# Change allhandled keyword to include three extra services, to check ordering
$runtime->storage->dbh_do(
           sub {
             my ($storage, $dbh) = @_;
             $dbh->do("insert into opsview_viewports values (?,?,?,?,?,?)",
                {}, 9,'allhandled','opslave','Nagios Stats', 137, 222 );
             $dbh->do("insert into opsview_viewports values (?,?,?,?,?,?)",
                {}, 9,'allhandled','monitored_by_slave','/', 136, 5 );
             $dbh->do("insert into opsview_viewports values (?,?,?,?,?,?)",
                {}, 9,'allhandled','monitored_by_slave','Check Loadavg', 136, 194 );
           },
         );

$admin->get_ok("/viewport/allhandled?output=json&style=errors_and_host_cells");
$expected = {
   'ResultSet' => {
     'list' => [
       {
         'acknowledged' => 1,
         'alias' => 'doesnt_exist_2',
         'current_check_attempt' => '1',
         'downtime' => 0,
         'icon' => 'debian',
         'last_check' => '1970-01-01 01:00:00',
         'max_check_attempts' => '3',
         'name' => 'doesnt_exist_2',
         'num_interfaces' => '0',
         'num_services' => '1',
         'output' => '',
         'services' => [
           {
             'acknowledged' => 1,
             'acknowledged_author' => undef,
             'acknowledged_comment' => undef,
             'current_check_attempt' => '1',
             'downtime' => 0,
             'last_check' => '2007-10-31 13:20:40',
             'markdown' => '0',
             'max_check_attempts' => '3',
             'name' => 'TCP/IP',
             'output' => 'forced result',
             'perfdata_available' => '1',
             'service_object_id' => '217',
             'state' => 'unknown',
             'unhandled' => '0'
           }
         ],
         'state' => 'down',
         'summary' => {
           'computed_state' => 'unknown',
           'handled' => 1,
           'total' => 1,
           'unhandled' => 0,
           'unknown' => 1
         },
         'unhandled' => '0'
       },
       {
         'alias' => 'Host to be monitored by slave',
         'current_check_attempt' => '0',
         'downtime' => 0,
         'flapping' => 1,
         'icon' => 'vmware',
         'last_check' => '1970-01-01 00:00:00',
         'max_check_attempts' => '0',
         'name' => 'monitored_by_slave',
         'num_interfaces' => '0',
         'num_services' => '5',
         'output' => 'Dummy output',
         'services' => [
           {
             'current_check_attempt' => '1',
             'downtime' => 1,
             'downtime_comment' => 'Hostgroup \'Leaf2\': in the future <script>alert("badness")</script>',
             'downtime_username' => 'admin',
             'last_check' => '2007-10-31 13:18:40',
             'markdown' => '0',
             'max_check_attempts' => '3',
             'name' => 'VNC',
             'output' => 'TCP CRITICAL - Invalid hostname, address or socket: monitored_by_slave',
             'perfdata_available' => '0',
             'service_object_id' => '5',
             'state' => 'critical',
             'unhandled' => '1'
           }
         ],
         'state' => 'up',
         'summary' => {
           'computed_state' => 'critical',
           'critical' => 1,
           'handled' => 0,
           'total' => 1,
           'unhandled' => 1
         },
         'unhandled' => '0'
       },
       {
         'alias' => 'Slave',
         'current_check_attempt' => '0',
         'downtime' => 0,
         'icon' => 'opsview',
         'last_check' => '1970-01-01 00:00:00',
         'max_check_attempts' => '0',
         'name' => 'opslave',
         'num_interfaces' => '0',
         'num_services' => '6',
         'output' => 'Dummy output',
         'services' => [
           {
             'current_check_attempt' => '1',
             'downtime' => 0,
             'last_check' => '2011-01-16 23:16:33',
             'markdown' => '0',
             'max_check_attempts' => '3',
             'name' => 'Nagios Stats',
             'output' => 'fake nagios stats for opsview',
             'perfdata_available' => '1',
             'service_object_id' => '222',
             'state' => 'critical',
             'unhandled' => '1'
           }
         ],
         'state' => 'up',
         'summary' => {
           'computed_state' => 'critical',
           'critical' => 1,
           'handled' => 0,
           'total' => 1,
           'unhandled' => 1
         },
         'unhandled' => '0'
       },
       {
         'alias' => 'Host to be monitored by slave',
         'current_check_attempt' => '0',
         'downtime' => 0,
         'flapping' => 1,
         'icon' => 'vmware',
         'last_check' => '1970-01-01 00:00:00',
         'max_check_attempts' => '0',
         'name' => 'monitored_by_slave',
         'num_interfaces' => '0',
         'num_services' => '5',
         'output' => 'Dummy output',
         'services' => [
           {
             'current_check_attempt' => '1',
             'downtime' => 1,
             'downtime_comment' => 'Hostgroup \'Leaf2\': in the future <script>alert("badness")</script>',
             'downtime_username' => 'admin',
             'last_check' => '2007-10-31 13:20:40',
             'markdown' => '0',
             'max_check_attempts' => '3',
             'name' => 'Check Loadavg',
             'output' => 'Invalid host name \'monitored_by_slave\'',
             'perfdata_available' => '0',
             'service_object_id' => '194',
             'state' => 'unknown',
             'unhandled' => '1'
           }
         ],
         'state' => 'up',
         'summary' => {
           'computed_state' => 'unknown',
           'handled' => 0,
           'total' => 1,
           'unhandled' => 1,
           'unknown' => 1
         },
         'unhandled' => '0'
       },
     ],
     'summary' => {
       'handled' => 4,
       'host' => {
         'down' => 1,
         'handled' => 3,
         'total' => 3,
         'unhandled' => 0,
         'up' => 2
       },
       'service' => {
         'critical' => 2,
         'handled' => 1,
         'total' => 4,
         'unhandled' => 3,
         'unknown' => 2
       },
       'total' => 7,
       'unhandled' => 3
     }
   }
 };

$obj = from_json( $admin->content );
Opsview::Test->strip_field_from_hash( "state_duration", $obj);
is_deeply( $obj, $expected, "Object as expected" ) || diag explain $obj;

